Package: smcfcs
Title: Multiple Imputation of Covariates by Substantive Model
        Compatible Fully Conditional Specification
Version: 2.0.2
Authors@R: c(person("Jonathan", "Bartlett", email = "jonathan.bartlett1@lshtm.ac.uk",role = c("aut", "cre")),
    person("Ruth", "Keogh", email = "ruth.keogh@lshtm.ac.uk",role = c("aut")),
    person("Edouard F.", "Bonneville",role = c("aut")),
    person("Lars", "van der Burg",role = c("aut")),
    person("Claus Thorn", "Ekstrøm",role = c("ctb")))
URL: https://github.com/jwb133/smcfcs
Description: Implements multiple imputation of missing covariates by
    Substantive Model Compatible Fully Conditional Specification.
    This is a modification of the popular FCS/chained equations
    multiple imputation approach, and allows imputation of missing
    covariate values from models which are compatible with the user
    specified substantive model.
Depends: R (>= 3.1.2)
License: GPL-3
LazyData: true
Imports: MASS, survival, VGAM, stats, rlang, checkmate, abind, brglm2
Suggests: knitr, rmarkdown, mitools, ggplot2, kmi, flexsurv, stringr
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
BugReports: https://github.com/jwb133/smcfcs/issues
NeedsCompilation: no
Packaged: 2026-04-30 11:43:27 UTC; lshjb12
Author: Jonathan Bartlett [aut, cre],
  Ruth Keogh [aut],
  Edouard F. Bonneville [aut],
  Lars van der Burg [aut],
  Claus Thorn Ekstrøm [ctb]
Maintainer: Jonathan Bartlett <jonathan.bartlett1@lshtm.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-01 05:10:02 UTC
