# Generated by roxygen2: do not edit by hand

S3method(plot,smcfcs)
export(smcfcs)
export(smcfcs.casecohort)
export(smcfcs.dtsam)
export(smcfcs.finegray)
export(smcfcs.flexsurv)
export(smcfcs.nestedcc)
export(smcfcs.parallel)
import(stats)
importFrom(rlang,.data)
importFrom(survival,Surv)
